﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using AttachedBehaviors.Framework;

namespace AttachedBehaviors.Usage
{
	public static class NullVisibility
	{
		public static readonly DependencyProperty ValueProperty = DependencyProperty.RegisterAttached(
			"Value",
			typeof(object),
			typeof(NullVisibility),
			new PropertyMetadata(true, OnArgumentsChanged));

		public static readonly DependencyProperty WhenNullProperty = DependencyProperty.RegisterAttached(
			"WhenNull",
			typeof(Visibility),
			typeof(NullVisibility),
			new PropertyMetadata(Visibility.Collapsed, OnArgumentsChanged));

		public static readonly DependencyProperty WhenNotNullProperty = DependencyProperty.RegisterAttached(
			"WhenNotNull",
			typeof(Visibility),
			typeof(NullVisibility),
			new PropertyMetadata(Visibility.Visible, OnArgumentsChanged));

		private static readonly AttachedBehavior Behavior =
			AttachedBehavior.Register(host => new NullVisibilityBehavior(host));

		private static void OnArgumentsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			Behavior.Update(d);
		}

		public static object GetValue(UIElement uiElement)
		{
			return uiElement.GetValue(ValueProperty);
		}

		public static void SetValue(UIElement uiElement, object value)
		{
			uiElement.SetValue(ValueProperty, value);
		}

		public static Visibility GetWhenNull(UIElement uiElement)
		{
			return (Visibility) uiElement.GetValue(WhenNullProperty);
		}

		public static void SetWhenNull(UIElement uiElement, Visibility visibility)
		{
			uiElement.SetValue(WhenNullProperty, visibility);
		}

		public static Visibility GetWhenNotNull(UIElement uiElement)
		{
			return (Visibility) uiElement.GetValue(WhenNotNullProperty);
		}

		public static void SetWhenNotNull(UIElement uiElement, Visibility visibility)
		{
			uiElement.SetValue(WhenNotNullProperty, visibility);
		}

		private sealed class NullVisibilityBehavior : Behavior<UIElement>
		{
			internal NullVisibilityBehavior(DependencyObject host) : base(host)
			{}

			protected override void Update(UIElement host)
			{
				host.Visibility = GetValue(host) == null
					? GetWhenNull(host)
					: GetWhenNotNull(host);
			}
		}
	}
}